/*
 *  FatFs for TOPPERS
 *      FAT File system/
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 * 
 *  Copyright (C) 2005- by Industrial Technology Institute,
 *                          Miyagi Prefectural Government, JAPAN
 *  Copyright (C) 2007-2010 by 
 *                     GJ Business Division RICOH COMPANY,LTD. JAPAN
 *  Copyright (C) 2015-2017 by TOPPERS PROJECT Educational Group.
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: diskio.c,v 1.3 2017/07/06 16:24:36 roi Exp $
 */

/*
 *  FatFsꤷƤ벼̥쥤I/F
 *  ʣɥ饤֤б٤ΥåȥܡɤǤ
 *  ɥ饤֤ϣĤΤߥݡȤƤ롣
 */

#include "kernel_impl.h"
#include <string.h>	/*  memcpy()  */
#include <t_syslog.h>
#include <t_stdlib.h>
#include "storagedevice.h"
#include "diskio.h"

#define RETRY_COUNT    8

/*
 *  ǥɥ饤֤ν
 *
 * BYTE Drive	ʪɥ饤ֹ
 */
DSTATUS
disk_initialize(BYTE Drive)
{
	StorageDevice_t *psdev = SDMGetStorageDevice(Drive);

	if(psdev == NULL || psdev->pdevf == NULL || psdev->pdevf->_sdevf_diskinit == NULL)
		return STA_NODISK;
	return psdev->pdevf->_sdevf_diskinit(psdev);
}


/*
 *  ǥɥ饤֤ξּ
 *
 * BYTE Drive	ʪɥ饤ֹ
 */
DSTATUS
disk_status(BYTE Drive)
{
	StorageDevice_t *psdev = SDMGetStorageDevice(Drive);

	if(psdev == NULL || psdev->pdevf == NULL || psdev->pdevf->_sdevf_diskstatus == NULL)
		return STA_NODISK;
	return psdev->pdevf->_sdevf_diskstatus(psdev);
}


/*
 *  ǥɤ߹
 *
 * BYTE Driveʪɥ饤ֹ
 * BYTE* Buffer        ɤ߽ФХåեؤΥݥ
 * DWORD SectorNumber  ɤ߽Фϥֹ
 * BYTE SectorCount    ɤ߽Ф
 */
DRESULT
disk_read(BYTE Drive, BYTE* Buffer, DWORD SectorNumber, BYTE SectorCount)
{
	StorageDevice_t *psdev = SDMGetStorageDevice(Drive);

	if(psdev == NULL || psdev->pdevf == NULL || psdev->pdevf->_sdevf_diskread == NULL)
		return RES_ERROR;
	return psdev->pdevf->_sdevf_diskread(psdev, Buffer, SectorNumber, SectorCount);
}

/*
 *  ǥؤν񤭹
 *
 * BYTE Driveʪɥ饤ֹ
 * BYTE* Buffer        񤭹ǡؤΥݥ
 * DWORD SectorNumber  񤭹߳ϥֹ
 * BYTE SectorCount    񤭹ߥ
 */
DRESULT
disk_write(BYTE Drive, const BYTE* Buffer, DWORD SectorNumber, BYTE SectorCount)
{
	StorageDevice_t *psdev = SDMGetStorageDevice(Drive);

	if(psdev == NULL || psdev->pdevf == NULL || psdev->pdevf->_sdevf_diskwrite == NULL)
		return RES_ERROR;
	return psdev->pdevf->_sdevf_diskwrite(psdev, Buffer, SectorNumber, SectorCount);
}

DRESULT
disk_ioctl(BYTE Drive, BYTE Func, void* Buffer)
{
	StorageDevice_t *psdev = SDMGetStorageDevice(Drive);

	if(psdev == NULL || psdev->pdevf == NULL || psdev->pdevf->_sdevf_diskioctl == NULL)
		return RES_ERROR;
	return psdev->pdevf->_sdevf_diskioctl(psdev, Func, Buffer);
}

/*
 *  աμ
 *  ToDo̤
 */
DWORD
get_fattime(void)
{
	SYSTIM systim;
	DWORD  fdate = ((2017-1980)<<25)+(1<<21)+(1<<16);
	DWORD  ftime;

	if(SDeviceHead._get_datetime != 0)
		return (DWORD)SDeviceHead._get_datetime();
	else{
		get_tim(&systim);
		systim = (systim+1000L) /2000L;
		ftime  = systim % 30;
		ftime += ((systim/30) % 60)<<5;
		ftime += ((systim/(60*30))<<11);
		return (fdate+ftime);
	}
}

