/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2019 by TOPPERS PROJECT Educational Working Group.
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: fpgai2c.h 59 2019-04-18 17:47:51Z roi $
 */

/*
 *  CycloneV FPGA I2Cデバイスドライバの外部宣言
 */

#ifndef _FPGAI2C_H_
#define _FPGAI2C_H_

#include "fpga.h"
#include "socfpga_cv.h"

#ifdef __cplusplus
 extern "C" {
#endif

/*
 *  FPGAI2Cポート定義
 */
#define FPGAI2C1_PORTID         1
#define NUM_FPGAI2CPORT         1


#define I2C_FLAGS_OE            0x00000001


#ifndef TOPPERS_MACRO_ONLY

/*
 *  拡張UARTハンドラ定義
 */
typedef struct fpgai2c_struct FPGAI2C_Handle_t;
struct fpgai2c_struct {
	uint32_t    base;				/* I2C register base address */
	uint32_t	portid;				/* I2C ポートID */
	uint32_t    clkCount;			/* I2C clockカウンタ */
	uint32_t    flags;				/* I2C 設定フラグ */
	uint32_t    timeout;			/* I2C タイムアウト値(ms) */
	int         semlock;			/* I2C ロックセマフォ値 */
};

#endif /* TOPPERS_MACRO_ONLY */

/*
 *  拡張I2C関数のプロトタイプ宣言
 */
#ifndef TOPPERS_MACRO_ONLY

extern FPGAI2C_Handle_t *fpgai2c_init(ID portid, uint32_t speed);
extern ER fpgai2c_deinit(FPGAI2C_Handle_t *hi2c);
extern ER fpgai2c_memwrite(FPGAI2C_Handle_t *hi2c, uint16_t devaddr, uint16_t addr, uint16_t addrsize, const void * data, uint32_t Size);
extern ER fpgai2c_memread(FPGAI2C_Handle_t *hi2c, uint16_t devaddr, uint16_t addr, uint16_t addrsize, void * data, uint32_t Size);


#endif /* TOPPERS_MACRO_ONLY */

#ifdef __cplusplus
}
#endif

#endif	/* _FPGAI2C_H_ */

