/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 * 
 *  Copyright (C) 2008-2011 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2016-2017 by TOPPERS PROJECT Educational Working Group.
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: qspi.h 698 2016-12-20 18:25:10Z roi $
 */
/*
 * 
 *  CycloneV HPS QSPIデバイスドライバの外部宣言
 *
 */

#ifndef __QSPI_H__
#define __QSPI_H__

//#include "hwlib.h"

#ifdef __cplusplus
extern "C"
{
#endif  /* __cplusplus */

/*
 *  QSPIポート定義
 */
#define QSPI1_PORTID            1
#define NUM_QSPIPORT            1

/*
 *  QSPIオペレーション・モード定義
 */
#define QSPI_MODE_SINGLE        0	/* Use Standard Single SPI (SIO-SPI) mode */
#define QSPI_MODE_DUAL          1	/* Use Dual SPI (DIO-SPI) SPI mode */
#define QSPI_MODE_QUAD          2	/* Use Dual SPI (QIO-SPI) SPI mode */
#define MAX_QSPI_MODE           QSPI_MODE_QUAD

/*
 *  CSモード設定
 */
#define QSPI_CS_MODE_SINGLE     0	/* Select 1 of 4 chip select ss_n[3:0] */
#define QSPI_CS_MODE_DECODE     1	/* Select external 4 to 16 decode of ss_n[3:0]. */

/*
 *  フラッシュ・コマンド
 */
#define QSPI_STIG_OPCODE_WR_STAT_REG          (0x01)
#define QSPI_STIG_OPCODE_PP                   (0x02)
#define QSPI_STIG_OPCODE_READ                 (0x03)
#define QSPI_STIG_OPCODE_WRDIS                (0x04)
#define QSPI_STIG_OPCODE_RD_STAT_REG          (0x05)
#define QSPI_STIG_OPCODE_WREN                 (0x06)
#define QSPI_STIG_OPCODE_FASTREAD             (0x0B)
#define QSPI_DEVSZ_BYTESPERSUBSECTOR_RESET    (0x10)
#define QSPI_STIG_OPCODE_4BYTE_WRITE          (0x12)
#define QSPI_STIG_OPCODE_4BYTE_READ           (0x13)
#define QSPI_STIG_OPCODE_SUBSEC_ERASE         (0x20)
#define QSPI_STIG_OPCODE_4BYTE_SEC_ERASE      (0x21)
#define QSPI_STIG_OPCODE_QUAD_PP              (0x32)
#define QSPI_STIG_OPCODE_4QUAD_WRITE          (0x34)
#define QSPI_STIG_OPCODE_FASTREAD_DUAL_OUTPUT (0x3B)
#define QSPI_STIG_OPCODE_4BYTE_BLOCK_ERASE    (0x5C)
#define QSPI_STIG_OPCODE_CHIP_ERASE           (0x60)
#define QSPI_STIG_OPCODE_FASTREAD_QUAD_OUTPUT (0x6B)
#define QSPI_STIG_OPCODE_RDID                 (0x9F)
#define QSPI_STIG_OPCODE_DUAL_PP              (0xA2)
#define QSPI_STIG_OPCODE_ENTER_4BYTE_MODE     (0xB7)
#define QSPI_STIG_OPCODE_FASTREAD_DUAL_IO     (0xBB)
#define QSPI_STIG_OPCODE_DIE_ERASE            (0xC4)
#define QSPI_STIG_OPCODE_WR_EXT_REG           (0xC5)
#define QSPI_STIG_OPCODE_BULK_ERASE           (0xC7)
#define QSPI_STIG_OPCODE_RD_EXT_REG           (0xC8)
#define QSPI_STIG_OPCODE_SEC_ERASE            (0xD8)
#define QSPI_STIG_OPCODE_4BYTE_Q_SEC_ERASE    (0xDC)
#define QSPI_STIG_OPCODE_EXIT_4BYTE_MODE      (0xE9)
#define QSPI_STIG_OPCODE_FASTREAD_QUAD_IO     (0xEB)
#define QSPI_STIG_OPCODE_4QUAD_READ           (0xEC)

/*
 *  フラッシュコマンド Micron依存
 */
#define QSPI_STIG_OPCODE_N25Q512A_CLRFLGSR    (0x50)
#define QSPI_STIG_OPCODE_N25Q512A_DISCVR_PARAM  (0x5A)
#define QSPI_STIG_OPCODE_N25Q128              (0x61)
#define QSPI_STIG_REGISTER_N25Q128_VECR       (0x65)
#define QSPI_STIG_OPCODE_N25Q512A_RESET_EN    (0x66)
#define QSPI_STIG_OPCODE_N25Q512A_RDFLGSR     (0x70)
#define QSPI_STIG_OPCODE_N25Q512A_RESET_MEM   (0x99)

/*
 *  マニュファクチャID
 */
#define QSPI_STIG_RDID_JEDECID_MICRON         (0x20)
#define QSPI_STIG_RDID_JEDECID_SPANSION       (0x01)
#define QSPI_STIG_RDID_JEDECID_MACRONIX       (0xC2)


/*
 *  オンボードSRAMサイズ
 */
#define QSPI_SRAM_FIFO_SIZE     512

/*
 * The size of the onboard SRAM in entries. Each entry is word (32-bit) sized.
 */
#define QSPI_SRAM_FIFO_ENTRY_COUNT    (QSPI_SRAM_FIFO_SIZE / sizeof(uint32_t))



#define QSPI_ERROR_NONE     0x00000000		/* No error              */
#define QSPI_ERROR_UNFLOW   0x00000002		/* under flow */
#define QSPI_ERROR_RDREJECT 0x00000008		/* read reject */
#define QSPI_ERROR_WPROTECT 0x00000010		/* Protected Area Write Attempt */
#define QSPI_ERROR_IRGAHB   0x00000020		/* Illegal AHB Access Detected */
#define QSPI_ERROR_XFRLVL   0x00000040		/* Transfer Watermark Reached */
#define QSPI_ERROR_RXOVER   0x00000080		/* Receive Overflow */

#define QSPI_MAX_ERASE_CMDS  	4


typedef ER (*QSPI_INIT_FUNC_T)(void *h);
typedef ER (*QSPI_WAIT_FUNC_T)(void *h, uint32_t timeout);

/*
 *  QSPIデバイス設定構造体
 */
typedef struct QSPI_Init
{
	uint16_t                manuf_id;
    uint16_t                type_capacity;
    uint32_t                clk_phase;
    uint32_t                clk_pol;
    uint32_t                max_freq_mhz;
    uint32_t                tshsl_ns;
    uint32_t                tsd2d_ns;
    uint32_t                tchsh_ns;
    uint32_t                tslch_ns;
    uint32_t                page_size;
    uint32_t                addr_size;
    uint8_t                 inst_type;
    uint32_t                read_op_code;
    uint8_t                 read_addr_xfer_type;
    uint8_t                 read_data_xfer_type;
    uint32_t                read_dummy_cycles;
    uint32_t                write_op_code;
    uint8_t                 write_addr_xfer_type;
    uint8_t                 write_data_xfer_type;
    uint32_t                write_dummy_cycles;
    uint8_t                 erase_count;
    uint32_t                erase_sizes[QSPI_MAX_ERASE_CMDS];
    uint8_t                 erase_cmds[QSPI_MAX_ERASE_CMDS];
    uint8_t                 erase_sector_idx;
    bool_t                  support_chip_erase;
    uint32_t                device_size;
    bool_t                  micron_multi_die;
    uint32_t                die_size;
    QSPI_INIT_FUNC_T        init_func;
    QSPI_WAIT_FUNC_T        wait_func;
    const char              *friendly_name;
} QSPI_Init_t;

/*
 *   QSPIデバイスハンドラ定義
 */
typedef struct _QSPI_Handle_t QSPI_Handle_t;
struct _QSPI_Handle_t
{
	uint32_t              base;			/* QSPI レジスタベースアドレス */
	uint32_t              clk_freq;		/* QSPI base clock */
	QSPI_Init_t           Init;			/* QSPI FLASHデータ */
	uint32_t              *pBuffPtr;	/* QSPI 通信バッファへのポインタ */
	uint32_t              XferSize;		/* QSPI 通信サイズ */
	volatile uint32_t     XferCount;	/* QSPI 通信カウンタ */
	volatile uint32_t     istatus;		/* QSPI 割込みステータス */
	int                   semid;		/* QSPI 通信用セマフォ値 */
	volatile uint32_t     errorcode;	/* QSPI エラーコード */
};

extern QSPI_Handle_t *qspi_init(ID portid, QSPI_Init_t *pini);
extern ER qspi_deinit(QSPI_Handle_t *hqspi);
extern ER qspi_read(QSPI_Handle_t *hqspi, void * dest, uint32_t src, uint32_t size);
extern ER qspi_write(QSPI_Handle_t *hqspi, uint32_t dest, const void * src, uint32_t size);
extern ER qspi_erase(QSPI_Handle_t *hqspi, uint32_t address, uint32_t size); 
extern ER qspi_erase_exec(QSPI_Handle_t *hqspi, uint32_t address, uint32_t idx);
extern void qspi_direct_disable(QSPI_Handle_t *hqspi);
extern void qspi_direct_enable(QSPI_Handle_t *hqspi, const uint32_t ahb_remap_addr);
extern uint32_t qspi_ahb_remap_address_get(QSPI_Handle_t *hqspi);
extern void qspi_hps_handler(ID portid);

extern ER qspi_chip_select_config_get(QSPI_Handle_t *hqspi, uint32_t *pcs, uint8_t *pcs_mode);
extern ER qspi_chip_select_config_set(QSPI_Handle_t *hqspi, const uint32_t cs, const uint8_t cs_mode);
extern void qspi_mode_bit_disable(QSPI_Handle_t *hqspi);
extern void qspi_mode_bit_enable(QSPI_Handle_t *hqspi);
extern ER qspi_mode_bit_config_set(QSPI_Handle_t *hqspi, const uint32_t mode_bits);

extern ER qspi_sram_partition_set(QSPI_Handle_t *hqspi, const uint32_t read_part_size);


/*
 *  CHIP消去
 *  parameter1 hqspi   QSPIハンドラへのポインタ
 *  return             正常終了時、E_OK
 */
#define qspi_erase_chip(h) qspi_erase(h, 0, (h)->Init.device_size)


/*
 *  セクター消去
 *  parameter1 hqspi   QSPIハンドラへのポインタ
 *  parameter2 address セクター先頭アドレス
 *  return             正常終了時、E_OK
 */
#define qspi_erase_sector(h, a) qspi_erase_exec(h, a, (h)->Init.erase_sector_idx)

/*
 *  セクターサイズを取り出す
 *  parameter1 hqspi   QSPIハンドラへのポインタ
 *  return             セクターバイトサイズ
 */
#define get_smallest_sector_size(h) (h)->Init.erase_sizes[(h)->Init.erase_sector_idx]


/*
 *  デバイスサイズを取り出す
 *  parameter1 hqspi   QSPIハンドラへのポインタ
 *  return             デバイスバイトサイズ
 */
#define qspi_get_device_size(h)  (h)->Init.device_size;

/*
 *  ページサイズを取り出す
 *  parameter1 hqspi   QSPIハンドラへのポインタ
 *  return             ページバイトサイズ
 */
#define qspi_get_page_size(h) (h)->Init.page_size

/*
 *  ダイページサイズを取り出す
 *  parameter1 hqspi   QSPIハンドラへのポインタ
 *  return             ダイバイトサイズ
 */
#define qspi_get_die_size(h) (h)->Init.micron_multi_die ? (h)->Init.die_size : (h)->Init.device_size



extern ER qspi_sr_wait_write(void *h, uint32_t timeout);
extern ER qspi_stig_cmd(QSPI_Handle_t *hqspi, uint32_t reg_value, uint32_t timeout);
extern ER qspi_stig_rd_cmd(QSPI_Handle_t *hqspi, uint8_t opcode, uint32_t dummy, uint32_t num_bytes, uint32_t * output, uint32_t timeout);
extern ER qspi_stig_wr_cmd(QSPI_Handle_t *hqspi, uint8_t opcode, uint32_t dummy, uint32_t num_bytes, const uint32_t * input, uint32_t timeout);



#ifdef __cplusplus
}
#endif  /* __cplusplus */
#endif  /* __QSPI_H__ */

