/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 * 
 *  Copyright (C) 2008-2011 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2017 by TOPPERS PROJECT Educational Working Group.
 * 
 *  上記著作権者は，以下の (1)～(4) の条件か，Free Software Foundation 
 *  によって公表されている GNU General Public License の Version 2 に記
 *  述されている条件を満たす場合に限り，本ソフトウェア（本ソフトウェア
 *  を改変したものを含む．以下同じ）を使用・複製・改変・再配布（以下，
 *  利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，その適用可能性も
 *  含めて，いかなる保証も行わない．また，本ソフトウェアの利用により直
 *  接的または間接的に生じたいかなる損害に関しても，その責任を負わない．
 * 
 *  @(#) $Id: sdcard.h,v 1.0 2016/11/02 18:11:54 roi Exp $
 */
/*
 *  STM32F746用 SDMMCドライバインクルードファイル
 */

#ifndef _SDMMC_H_
#define _SDMMC_H_

#ifdef __cplusplus
 extern "C" {
#endif

#include "socfpga_cv.h"

/*
 *  SDMMC拡張エラー定義
 */
#define E_SDCOM             (-80)	/* コマンドエラー */


#define SD_CARD_V11         0
#define SD_CARD_V20         1
#define SD_CARD_HC          2
#define MMC_CARD			3
#define SD_IO_CARD          4
#define HS_MM_CARD          5
#define SD_IO_COMBO_CARD    6
#define MMC_CARD_HC         7

#define SD_CARD_NOTDETECT   32

/*
 *  バス幅定義
 */
#define SDMMC_BUS_WIDE_1B   1
#define SDMMC_BUS_WIDE_4B   4
#define SDMMC_BUS_WIDE_8B   8

#define SDMODE_PROTECT      (1<<5)

#define CMD_INDEX           0x003F

/*
 *  SD/MMC コマンドリスト
 */
#define MCI_CMD0			(0x00|0x2000a000)						/* GO_IDLE_STATE(MMC) or RESET(SD) */
#define MCI_CMD2			(0x02|0x200020c0)						/* ALL SEND_CID */
#define MCI_CMD3			(0x03|0x20002040)						/* SET_RELATE_ADDR */
#define MCI_CMD6			(0x06|0x20002240)						/* SEND SWITCH STATUS */
#define MCI_CMD7			(0x07|0x20002040)						/* SELECT/DESELECT_CARD */
#define MCI_CMD8            (0x08|0x20002040)						/* HS_SEND_EXT_CSD */
#define MCI_CMD9			(0x09|0x200020c0)						/* SEND_CSD */
#define MCI_CMD10			(0x0A|0x200020c0)						/* SEND_CID */
#define MCI_CMD12			(0x0C|0x20006000)						/* Stop either READ or WRITE operation */
#define MCI_CMD13			(0x0D|0x20000040)						/* SEND_STATUS */
#define MCI_CMD16			(0x10|0x20002040)						/* SET_BLOCK_LEN */
#define MCI_CMD17			(0x11|0x20002240)						/* READ_SINGLE_BLOCK */
#define MCI_CMD18			(0x12|0x20003240)						/* READ_MULTI_BLOCK */
#define MCI_CMD24			(0x18|0x20002240|SDMMC_CMD_RD_WR)		/* WRITE_SINGLE_BLOCK */
#define MCI_CMD25			(0x19|0x20003240|SDMMC_CMD_RD_WR)		/* WRITE_MULTI_BLOCK */
#define MCI_CMD55			(0x37|0x20002040)						/* APP_CMD, the following will a ACMD */

#define MCI_ACMD6			(0x06|0x20002040)						/* ACMD6 for SD card BUSWIDTH */
#define MCI_ACMD13			(0x0D|0x20000240)						/* ACMD13 for SD card status */
#define MCI_ACMD41			(0x29|0x20002040)						/* ACMD41 for SD card */
#define MCI_ACMD51			(0x33|0x20000240)						/* ACMD51 for SD SCR */

#define MCI_CLK             0x0020203f								/* CLOCK COMAND */

#define RES_CMD1            (MCI_CMD1 & CMD_INDEX)
#define RES_CMD12           (MCI_CMD12 & CMD_INDEX)
#define RES_ACMD41          (MCI_ACMD41 & CMD_INDEX)

/*
 *  SDMMACステータス
 */
#define SDMMC_STATUS_NONE       0
#define SDMMC_STATUS_INIT       1
#define SDMMC_STATUS_READY      2
#define SDMMC_STATUS_ACTIVATE   3
/*
 *  SDMMCエラーコード
 */
#define SDMMC_ERROR_NONE        0x00000000			/* No error */
#define SDMMC_ERROR_RESPONE     SDMMC_INTSTS_RE		/* Response error */
#define SDMMC_ERROR_RCRC        SDMMC_INTSTS_RCRC	/* Response CRC error */
#define SDMMC_ERROR_DCRC        SDMMC_INTSTS_DCRC	/* Data CRC error */
#define SDMMC_ERROR_RTO         SDMMC_INTSTS_BAR	/* Response Timeout error */
#define SDMMC_ERROR_DRTO        SDMMC_INTSTS_BDS	/* Data Read Timeout error */
#define SDMMC_ERROR_FRUN        SDMMC_INTSTS_FRUN	/* Fifo underun error */
#define SDMMC_ERROR_HLE         SDMMC_INTSTS_HLE	/* Hardware Locked Write Error */
#define SDMMC_ERROR_SBE         SDMMC_INTSTS_SBE	/* Start-bit error */
#define SDMMC_ERROR_EBE         SDMMC_INTSTS_EBE	/* End-bit Error */

#define SDMMC_ERROR_MASK        ( SDMMC_ERROR_RESPONE | SDMMC_ERROR_RCRC | SDMMC_ERROR_DCRC |\
                                  SDMMC_ERROR_RTO     | SDMMC_ERROR_DRTO | SDMMC_ERROR_FRUN |\
                                  SDMMC_ERROR_HLE     | SDMMC_ERROR_SBE  | SDMMC_ERROR_EBE )

#define INHNO_SDMMC  (0x00010000 | INTNO_SDMMC_IRQ)	/* 割込みハンドラ番号 */
#define INTNO_SDMMC  INTNO_SDMMC_IRQ	/* 割込み番号 */
#define INTPRI_SDMMC -9			/* 割込み優先度 */
#define INTATR_SDMMC 0			/* 割込み属性 */

#ifndef TOPPERS_MACRO_ONLY

/*
 *  SD/MMC内部DMAコントローラ・ディスクリプタ
 */
typedef struct
{
	/*
	 *  The DES0 field in the internal DMA controller descriptor
	 *  contains control and status information.
	 */
	union DES0
	{
	    struct {
			uint32_t                  :  1;		/* Reserved */
			uint32_t  dic             :  1;		/* Disable Interrupt on Completion(DIC */
			uint32_t  ld              :  1;		/* Last Descriptor (LD). */
			uint32_t  fs              :  1;		/* First Descriptor (FS). */
			uint32_t  ch              :  1;		/* Second Address Chained(CH) */
			uint32_t  er              :  1;		/* End of Ring (ER) */
			uint32_t  ces             :  1;		/* Card Error Summary (CES) */
			uint32_t                  : 24;		/* Reserved */
			uint32_t  own             :  1;		/* indicates that the descriptor owned by the DMA */
		} fld;									/* Union data member access to DES0 fields */

		uint32_t  raw;							/* The DES0 raw register aggregate value */
	} des0;
	/*
	 * The DES1 descriptor field contains the buffer size.
	 */
	union DES1
	{
		struct {
			uint32_t  bs1             : 13;		/* Buffer 1 Size (BS1). */
			uint32_t  bs2             : 13;		/* Buffer 2 Size (BS2). */
			uint32_t                  :  6;		/* Reserved */
		} fld;									/* Union data member access to DES1 fields */

		uint32_t  raw;							/* The DES1 raw register aggregate value */
	} des1;

	/*
	 * The DES2 descriptor field contains the address pointer to the data buffer.
	 */
	union DES2
	{
		struct {
			uint32_t  bap1            : 32;		/* Buffer Address Pointer 1(BAP1) */
		} fld;									/* Union data member access to DES2 fields */

		uint32_t  raw;							/* The DES2 raw register aggregate value */
	} des2;

	/*
	 *  The DES3 descriptor field contains the address pointer to the next descriptor
	 */
	union DES3
	{
		struct {
			uint32_t  bap2_or_next    : 32;		/* Buffer Address Pointer 2 (BAP2) or Next Descriptor Address */
		} fld;									/* Union data member access to DES3 fields */

		uint32_t  raw;							/* The DES3 raw register aggregate value */
	} des3;
}SDMMC_DMA_BUF_DESC_t;


/*
 *  SDMMCハンドラ構造体定義
 */

typedef struct
{
	uint32_t                base;		/* SDMMCレジスタベースアドレス */
	uint32_t                ClockMode;	/* 指定クロックモード */
	uint32_t                BusWide;	/* 指定バス幅 */
	uint32_t                RetryCount;	/* リトライ回数 */
	uint32_t                cardtype;	/* カードタイプ */
	uint32_t                RCA;		/* RCA値 */
	uint32_t                CSD[4];		/* CSD値 */
	uint32_t                CID[4];		/* CID値 */
	uint32_t                SCR[2];		/* SCR値 */
    uint32_t                CCC;		/* CCC値 */
	uint32_t                block_size;	/* READ/WRITEブロックサイズ */
	bool_t                  high_speed;	/* 50MHz high speed許可 */
	bool_t                  dma_enable;	/* DMA有効無効 */
	uint8_t                 *pBuffPtr;	/* SDMMC 通信バッファへのポインタ */
	SDMMC_DMA_BUF_DESC_t    *pCurDesc;	/* 現在のDMAディスクリプタ */
	volatile uint32_t       XferCount;	/* SDMMC 通信カウンタ */
	volatile uint32_t       istatus;	/* 割込み状態 */
	volatile uint32_t       dstatus;	/* 割込みDMA状態 */
	volatile uint32_t       status;		/* 転送状態 */
	volatile uint32_t       SdCmd;		/* 転送指定コマンド */
	volatile uint32_t       ErrorCode;	/* エラーコード */
	ID                      semid;		/* セマフォID */
}SDMMC_Handle_t;

/*
 *  SDMMCカード情報構造体
 */
typedef struct
{
	uint64_t                capacity;	/* 容量(バイト) */
	uint32_t                blocksize;	/* ブロックサイズ */
	uint32_t                maxsector;	/* 最大セクタ数 */
	uint16_t                RCA;		/* SD RCA */
	uint8_t                 cardtype;	/* カード種類 */
	uint8_t                 status;		/* ステータス */

}SDMMC_CardInfo_t;



extern bool_t sdmmc_sense(int id);
extern void sdmmc_init(intptr_t exinf);
extern void sdmmc_deinit(int id);
extern bool_t sdmmc_card_is_write_protected(SDMMC_Handle_t *hsd);
extern SDMMC_Handle_t *sdmmc_open(int id);
extern ER sdmmc_close(SDMMC_Handle_t *hsd);
extern void sdmmc_hps_handler(void);

extern ER sdmmc_checkCID(SDMMC_Handle_t *hsd);
extern ER sdmmc_setaddress(SDMMC_Handle_t *hsd);
extern ER sdmmc_setaddress(SDMMC_Handle_t *hsd);
extern ER sdmmc_sendCSD(SDMMC_Handle_t *hsd);
extern ER sdmmc_getcardinfo(SDMMC_Handle_t *hsd, SDMMC_CardInfo_t *pCardInfo);
extern ER sdmmc_select_card(SDMMC_Handle_t *hsd, uint64_t addr);
extern ER sdmmc_configuration(SDMMC_Handle_t *hsd);
extern ER sdmmc_set_widebus(SDMMC_Handle_t *hsd);
extern ER sdmmc_blockread(SDMMC_Handle_t *hsd, void *dest, uint64_t src, uint32_t blocksize, uint32_t num);
extern ER sdmmc_blockwrite(SDMMC_Handle_t *hsd, void *src, uint64_t WriteAddr, uint32_t blocksize, uint32_t num);
extern ER sdmmc_wait_transfar(SDMMC_Handle_t *hsd, uint32_t Timeout);

#endif /* TOPPERS_MACRO_ONLY */

#ifdef __cplusplus
}
#endif


#endif /* _SDMMC_H_ */

