/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 * 
 *  Copyright (C) 2008-2011 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2017 by TOPPERS PROJECT Educational Working Group.
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: spi.h 698 2017-01-11 18:37:10Z roi $
 */
/*
 * 
 *  CycloneV HPS SPIデバイスドライバの外部宣言
 *
 */

#ifndef _SPI_H_
#define _SPI_H_

#include "device.h"

#ifdef __cplusplus
 extern "C" {
#endif

/*
 *  SPIポート定義
 */
#define SPI1_PORTID             1
#define SPI2_PORTID             2
#define SPIS1_PORTID            3
#define SPIS2_POERID            4
#define NUM_SPIPORT             4

/*
 *  SPIフレームモード設定
 */
#define SPI_FRM_SPI             0x00000000	/* Motorola SPI protocol */
#define SPI_FRM_SSP             0x00000010	/* Texas Instruments Serial Protocol (SSP) */
#define SPI_FRM_MICROWIRE       0x00000020	/* National Semiconductor Microwire */

/*
 *  SPI転送クロック極性定義
 */
#define SPI_POLARITY_LOW        0x00000000
#define SPI_POLARITY_HIGH       SPIM_CTLR0_SCPOL

/*
 *  SPIクロック位相定義
 */
#define SPI_PHASE_1EDGE         0x00000000
#define SPI_PHASE_2EDGE         SPIM_CTLR0_SCPH

/*
 *  SPI(MASTER)ループバックモード
 */
#define SPI_LOOPBACKOFF         0x00000000
#define SPI_LOOPBACKON          SPIM_CTLR0_SRL

/*
 *  SPI(SLAVE)アウトプットイネーブル
 */
#define SPI_SLAVEOUTDISABLE     0x00000000
#define SPI_SLAVEOUTENABLE      SPIS_CTLR0_SLV_OE

/*
 *  転送データ長指定
 */
#define MIN_SPI_DFS             4
#define MAX_SPI_DFS             16

/*
 *  SPIモード定義
 */
#define SPI_MODE_SLAVE          0		/* SPI Slave Controller */
#define SPI_MODE_MASTER         1		/* SPI Master Controller */

/*
 *  SPIマスターサンプルデレィ最大値
 */
#define SPI_RX_SAMPLE_DELAY_MAX 4

/*
 *  Microwire 転送モード定義
 */
#define SPI_MW_NON_SEQUENTIAL   0x00000000
#define SPI_MW_SEQUENTIAL       SPIM_MWCR_MWMOD

/*
 *  Microwire 転送方向定義
 */
#define SPI_MW_DIR_RX           0x00000000
#define SPI_MW_DIR_TX           SPIM_MWCR_MDD

/*
 *  Microwire ハンドシェーク設定
 */
#define SPI_MWHANDSHAKE_DISABLE 0x00000000
#define SPI_MWHANDSHAKE_ENABLE  SPIM_MWCR_MHS

/*
 *  Microwire Control Frame Size長指定
 */
#define MIN_SPI_MW_CTS_DFS      1
#define MAX_SPI_MW_CTS_DFS      16

/*
 *  転送モード設定
 */
#define SPI_XMOD_TXRX           0x00000000	/* 送受信モード */
#define SPI_XMOD_TX             0x00000100	/* 送信モード */
#define SPI_XMOD_RX             0x00000200	/* 受信モード */

/*
 *  SPIエラー定義
 */ 
#define SPI_ERROR_NONE          0x00000000		/* No error */
#define SPI_ERROR_TXFOVR        SPIS_ISR_TXOIS	/* TX FIFO Overflow */
#define SPI_ERROR_RXFUND        SPIS_ISR_RXUIS	/* RX FIFO Underflow */
#define SPI_ERROR_RXFOVR        SPIS_ISR_RXOIS	/* RX FIFO Overflow */
#define SPI_ERROR_DMA           0x00010000		/* DMA transfer error */
#define SPI_ERROR_TIMEOUT       0x00020000		/* タイムアウト */

/*
 *  SPIハードウェア設定構造体
 */

typedef struct _SPI_PortControlBlock{
	uint32_t              base;				/* SPIポートベースアドレス */
	uint32_t              intno;			/* 割込み番号 */
	uint32_t              mode;				/* SPIモード */
	uint32_t              resetbit;			/* SPIモジュールリセットビット */
} SPI_PortControlBlock;


/*
 *  SPI 設定初期設定構造体
 */
typedef struct
{
    uint32_t              DataSize;			/* SPI転送データサイズ */
	uint32_t              FrameMode;		/* SPI転送モード設定 */
	uint32_t              CLKPolarity;		/* SPI転送クロックの極性 */
	uint32_t              CLKPhase;			/* SPIクロック位相 */
	uint32_t              XferMode;			/* SPIデフォルト転送モード */
	uint32_t              Speed;			/* SPI転送スピード値 */
	uint32_t              RxSampleDelay;	/* SPI:MASTER 受信サンプルデレィ */
	uint32_t              LoopBack;			/* SPI:MASTER LOOPBACK MODE */
	uint32_t              SlaveOut;			/* SPI:SLAVE Output Enable */
	/*
	 *  Microwireフレームモード初期化パラメータ
	 */
	uint32_t              CtlFrameSize;		/* SPI:Microwire Control FrameSize */
	uint32_t              MwMode;			/* SPI:Microwire Transfer Mode */
	uint32_t              MwDir;			/* SPI:Microwire Transfer Direction */
	uint32_t              MwHandshake;		/* SPI:MASTER Microwire HandShaking */
	int                   semid;			/* SPI 通信用セマフォ値 */
	int                   semlock;			/* SPI ロックセマフォ値 */
} SPI_Init_t;


/*
 *  SPIハンドラ
 */
typedef struct
{
	uint32_t            base;				/* SPI registers base address */
	SPI_Init_t          Init;				/* SPI 初期化情報 */
	ID                  portid;				/* SPI ポート番号 */
    uint32_t            clock_freq;			/* SPI クロック値 */
	uint16_t            op_mode;			/* SPI マスタースレーブ設定 */
    uint16_t            slave_select;
    uint32_t            xmode;				/* SPI転送モード */
	uint8_t             *ptxbuff;			/* Pointer to SPI Tx transfer Buffer */
	uint32_t            txxfersize;			/* SPI Tx transfer size */
	uint32_t            txxfercount;		/* SPI Tx Transfer Counter */
	uint8_t             *prxbuff;			/* Pointer to SPI Rx transfer Buffer */
	uint32_t            rxxfersize;			/* SPI Rx transfer size */
	uint32_t            rxxfercount;		/* SPI Rx Transfer Counter */
	DMACH_Handle_t      *hdmachtx;			/* SPI Tx DMA Channel handle parameters */
	DMACH_Handle_t      *hdmachrx;			/* SPI Tx DMA Channel handle parameters */
	volatile uint32_t   errorcode;			/* SPI Error code */
} SPI_Handle_t;


extern SPI_Handle_t *spi_init(ID portid, const SPI_Init_t* cfg);
extern ER spi_deinit(SPI_Handle_t *hspi);
extern void spi_reset(SPI_Handle_t * hspi);
extern ER spi_transrecv(SPI_Handle_t *hspi, uint32_t slave_select, uint8_t *tx_buf, uint8_t *rx_buf, uint32_t len);
extern ER spi_transmit(SPI_Handle_t *hspi, uint32_t slave_select, uint8_t *tx_buf, uint32_t len);
extern ER spi_receive(SPI_Handle_t *hspi, uint32_t slave_select, uint8_t *rx_buf, uint32_t len);
extern void spim_hps_isr(intptr_t exinf);

extern ER spi_config_disp(SPI_Handle_t *hspi);

#ifdef __cplusplus
}
#endif

#endif	/* _SPI_H_ */

